% Simulation of Data Acquisition and Spectral Analysis
%
% SIMDAQ Illustrates the basic operations of data acquisition and
% spectral analysis.  Typically a real signal is first conditioned
% using an analog low-pass filter to reject any high-frequency noise
% before sampling.  So an appropriate filter is selected.  The
% filtered signal must be sampled at an adequate rate and duration
% to capture the characteristics of interest.  If the signal has
% great dynamic range, a window can be applied to the sampled data
% to reduce the interference of spectral leakage on small components
% of the signal.  If the signal has a significant DC offset, it is
% best to normalize the signal (subtract the mean) before windowing.
% These steps can clarify the spectrum for interpretation.
%
% SIGNAL mode is the operational display of the program.  The 
% upper axis shows the input signal, the filter output, and the
% samples located on the filter output.  The lower axis is for
% analysis of the data.  Several flavors of spectra are available,
% and also a signal reconstructed from the samples, with bandwidth
% limited by the sample rate.
% The popup menu labeled "Program Mode" gives access to the other
% modes, filter design and window selection.  The "Signal"
% popup menu selects one of the available input signal functions.
% SimDAQ requires a continuous input signal function same as
% Sampling.  The default signals are UnitSquare, UnitStep, Problem,
% and Noisy.  These are small functions that must be present in
% the program directory or on the Matlab search path.  Problem
% contains two different sinusoidal signals with a great disparity
% of amplitude, so when the stronger one is sampled off-picket,
% spectral leakage can overwhelm the weaker one.  Noisy contains
% a sinusoid signal and also a rectified sine of frequency 61.
% (the first nonzero harmonic is 122)
% The sampling parameters may be edited in their respective boxes
% as follows.  fs is the sampling frequency.  N is the number of
% samples.  Start is the time to take the first sample.  Warmup
% allows the analog filter to begin operation, hence "warming up,"
% some time before the start of sampling.  A real analog filter
% typically operates continuously, but a warmup time of 1/fc 
% is usually sufficient to reduce the effect of the filter
% startup transient and improve the fidelity of the sampled data.
% Of course, the UnitStep signal is included specifically to
% isolate the filter step response.
% The sampled filter output may be adjusted by removing the mean
% ("normalizing") and applying a window function, before the 
% spectrum is taken.  These intermediate steps are not shown.
% The popup menu labeled "Display" controls the format of the
% spectrum display.  The Decibel display is vulnerable to cases
% where some of the spectrum values are zero or extremely small.
% Zeros cause breaks in the display line.  Extremely small
% values cause an unreasonable vertical scale so the interesting
% parts of the spectrum are shrunken.  In this case activate the 
% zoom control and adjust the view.  The zoom control also applies
% to the upper axis.
% The forcing function, the sampled filter output, and the spectrum
% are assigned in the main workspace by the "Return" button.  If
% the spectrum display is Reconstruct, what is assigned in the
% main workspace is the modified data, after optional normalizing
% and windowing.
%
% In FILTER mode, a low-pass filter is specified and the filter
% performance graphs are shown.  The available filter types are
% "None," (no filter), Bessel, Butterworth, and Chebychev.  These
% filters are implemented as ordinary differential equations.  
% Another popular type of filter is the Elliptical or "Cauer" 
% filter, but it is fundamentally different, being governed by an
% integral-differential equation, so it could not be implemented
% in this program.  Filter type is selected using the popup menu.
% Filter order, cutoff frequency fc, and ripple (Chebychev only),
% are specified in edit boxes.  In another edit box labeled
% "Check f" the user can specify a frequency to calculate
% attenuation and phase delay, in order to verify filter 
% performance.  The "Filter Display" popup menu controls the
% format of the attenuation and phase delay displays.
% Note that even-order Chebychev filters have DC (zero frequency)
% attenuation of less than one because of the ripple.
%
% In WINDOW mode, a data window function is specifed and the
% window shape and spectral performance are shown.  The check box
% that activates data normalizing, which is done before windowing,
% is also located here.
% The window functions available in the "Window" popup menu are
% None (also known as the "Boxcar" window), Bartlett, Hanning,
% Hamming, and the Kaiser window with alpha=2.5 (same as beta=7.85
% in the Matlab toolbox version of Kaiser).  The exact window shape
% and spectral performance are affected by N and fs.  The lower
% right axis shows the whole spectrum kernel for the window.  This
% is the effect of spectral leakage to be expected when a signal is
% sampled off-picket.  The relative frequency axis is from -fs/2 to
% fs/2, where the zero of relative frequency corresponds to the
% exact signal frequency in the data spectrum.  In the lower left
% axis the relative frequency scale is magnified for a better view
% of the kernal main peak and side lobes, out to four times the
% spectrum frequency resolution.  Generally the more sophisticated
% windows reduce the side lobes and far field spectrum while
% broadening the main peak and also reducing its value from 1.
% Thus a peak of 1 in Bartlett-windowed data indicates an actual
% signal amplitude of approximately 2.
% The "Kernal Display" popup menu selects the vertical axis of
% the spectrum kernals, among magnitude, power or decibels.
%
% SIMDAQ('myfun','F') replaces the UnitSquare input signal with any
% other function name "F."  For example:
%        simdaq('myfun','cos')
% replaces the unit square wave with the cosine function.  Any 
% standard or user-defined function can be filtered and sampled 
% in this way.
%
% See also FOURSERIES, FUNSPECT, DATASPECT, FIRSTORD, 
% SECONDORD, MULTISYS, and SAMPLING.

% Copyright Tim Conover 2000
